@@ -50,8 +50,6 @@ G_BEGIN_DECLS
 #define TRACKER_IS_DB_RESULT_SET_CLASS(c)   (G_TYPE_CHECK_CLASS_TYPE ((o),    TRACKER_TYPE_DB_RESULT_SET))
 #define TRACKER_DB_RESULT_SET_GET_CLASS(o)  (G_TYPE_INSTANCE_GET_CLASS ((o),  TRACKER_TYPE_DB_RESULT_SET, TrackerDbResultSetClass))
 
-#define TRACKER_TYPE_DB_BLOB                (tracker_db_blob_get_type ())
-
 #define TRACKER_DB_INTERFACE_ERROR          (tracker_db_interface_error_quark ())
 
 typedef enum {
@@ -76,8 +74,6 @@ struct TrackerDBInterfaceIface {
 	TrackerDBResultSet * (* execute_query)    (TrackerDBInterface  *interface,
 	                                           GError             **error,
 	                                           const gchar         *query);
-	void                 (* disconnect)       (TrackerDBInterface  *interface);
-	void                 (* reconnect)        (TrackerDBInterface  *interface);
 };
 
 struct TrackerDBStatementIface {
@@ -126,6 +122,7 @@ struct TrackerDBCursorIface {
 	                                guint            column);
 	gdouble       (*get_double)    (TrackerDBCursor *cursor,
 	                                guint            column);
+
 };
 
 GQuark              tracker_db_interface_error_quark       (void);
@@ -134,7 +131,6 @@ GType               tracker_db_interface_get_type          (void);
 GType               tracker_db_statement_get_type          (void);
 GType               tracker_db_cursor_get_type             (void);
 GType               tracker_db_result_set_get_type         (void);
-GType               tracker_db_blob_get_type               (void);
 
 /* Functions to create queries/procedures */
 TrackerDBStatement *tracker_db_interface_create_statement  (TrackerDBInterface   *interface,
@@ -150,9 +146,7 @@ TrackerDBResultSet *tracker_db_interface_execute_query     (TrackerDBInterface
                                                             ...) G_GNUC_PRINTF (3, 4);
 
 gboolean            tracker_db_interface_start_transaction (TrackerDBInterface  *interface);
-gboolean            tracker_db_interface_end_transaction   (TrackerDBInterface  *interface);
-void                tracker_db_interface_disconnect        (TrackerDBInterface  *interface);
-void                tracker_db_interface_reconnect         (TrackerDBInterface  *interface);
+gboolean            tracker_db_interface_end_db_transaction   (TrackerDBInterface  *interface);
 void                tracker_db_statement_bind_double       (TrackerDBStatement  *stmt,
                                                             int                  index,
                                                             double               value);
