@@ -44,15 +44,17 @@
 /*
  * data_format:
  * #... 0000 0000 (total size is 4 bytes)
- *            ||`- resource insert (all other bits must be 0 if 1)
- *            |`-- object type (1 = id, 0 = cstring)
- *            `--- operation type (0 = insert, 1 = delete)
+ *           |||`- resource insert (all other bits must be 0 if 1)
+ *           ||`-- object type (1 = id, 0 = cstring)
+ *           |`--- operation type (0 = insert, 1 = delete)
+ *           `---- graph (0 = default graph, 1 = named graph)
  */
 
 typedef enum {
 	DATA_FORMAT_RESOURCE_INSERT  = 1 << 0,
 	DATA_FORMAT_OBJECT_ID        = 1 << 1,
-	DATA_FORMAT_OPERATION_DELETE = 1 << 2
+	DATA_FORMAT_OPERATION_DELETE = 1 << 2,
+	DATA_FORMAT_GRAPH            = 1 << 3
 } DataFormat;
 
 static struct {
@@ -68,6 +70,7 @@ static struct {
 	gint64 time;
 	TrackerDBJournalEntryType type;
 	const gchar *uri;
+	gint g_id;
 	gint s_id;
 	gint p_id;
 	gint o_id;
@@ -190,7 +193,7 @@ tracker_db_journal_error_quark (void)
 }
 
 gboolean
-tracker_db_journal_init (const gchar *filename)
+tracker_db_journal_init (const gchar *filename, gboolean truncate)
 {
 	gchar *directory;
 	struct stat st;
@@ -235,6 +238,14 @@ tracker_db_journal_init (const gchar *filename)
 
 	mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP;
 	flags = O_WRONLY | O_APPEND | O_CREAT;
+	if (truncate) {
+		/* existing journal contents are invalid: reindex where journal
+		 * does not even contain a single valid entry
+		 *
+		 * or should be ignored: only for test cases
+		 */
+		flags |= O_TRUNC;
+	}
 	writer.journal = g_open (writer.journal_filename, flags, mode);
 
 	if (writer.journal == -1) {
@@ -329,7 +340,7 @@ tracker_db_journal_start_transaction (time_t time)
 
 	/* Leave space for size, amount and crc
 	 * Check and keep in sync the offset variable at
-	 * tracker_db_journal_commit_transaction too */
+	 * tracker_db_journal_commit_db_transaction too */
 
 	memset (writer.cur_block, 0, size);
 
@@ -345,7 +356,8 @@ tracker_db_journal_start_transaction (time_t time)
 }
 
 gboolean
-tracker_db_journal_append_delete_statement (gint         s_id,
+tracker_db_journal_append_delete_statement (gint         g_id,
+                                            gint         s_id,
                                             gint         p_id,
                                             const gchar *object)
 {
@@ -354,18 +366,26 @@ tracker_db_journal_append_delete_statement (gint         s_id,
 	gint size;
 
 	g_return_val_if_fail (writer.journal > 0, FALSE);
+	g_return_val_if_fail (g_id >= 0, FALSE);
 	g_return_val_if_fail (s_id > 0, FALSE);
 	g_return_val_if_fail (p_id > 0, FALSE);
 	g_return_val_if_fail (object != NULL, FALSE);
 
 	o_len = strlen (object);
-	df = DATA_FORMAT_OPERATION_DELETE;
-
-	size = (sizeof (guint32) * 3) + o_len + 1;
+	if (g_id == 0) {
+		df = DATA_FORMAT_OPERATION_DELETE;
+		size = (sizeof (guint32) * 3) + o_len + 1;
+	} else {
+		df = DATA_FORMAT_OPERATION_DELETE | DATA_FORMAT_GRAPH;
+		size = (sizeof (guint32) * 4) + o_len + 1;
+	}
 
 	cur_block_maybe_expand (size);
 
 	cur_setnum (writer.cur_block, &writer.cur_pos, df);
+	if (g_id > 0) {
+		cur_setnum (writer.cur_block, &writer.cur_pos, g_id);
+	}
 	cur_setnum (writer.cur_block, &writer.cur_pos, s_id);
 	cur_setnum (writer.cur_block, &writer.cur_pos, p_id);
 	cur_setstr (writer.cur_block, &writer.cur_pos, object, o_len);
@@ -377,7 +397,8 @@ tracker_db_journal_append_delete_statement (gint         s_id,
 }
 
 gboolean
-tracker_db_journal_append_delete_statement_id (gint s_id,
+tracker_db_journal_append_delete_statement_id (gint g_id,
+                                               gint s_id,
                                                gint p_id,
                                                gint o_id)
 {
@@ -385,16 +406,25 @@ tracker_db_journal_append_delete_statement_id (gint s_id,
 	gint size;
 
 	g_return_val_if_fail (writer.journal > 0, FALSE);
+	g_return_val_if_fail (g_id >= 0, FALSE);
 	g_return_val_if_fail (s_id > 0, FALSE);
 	g_return_val_if_fail (p_id > 0, FALSE);
 	g_return_val_if_fail (o_id > 0, FALSE);
 
-	df = DATA_FORMAT_OPERATION_DELETE | DATA_FORMAT_OBJECT_ID;
-	size = sizeof (guint32) * 4;
+	if (g_id == 0) {
+		df = DATA_FORMAT_OPERATION_DELETE | DATA_FORMAT_OBJECT_ID;
+		size = sizeof (guint32) * 4;
+	} else {
+		df = DATA_FORMAT_OPERATION_DELETE | DATA_FORMAT_OBJECT_ID | DATA_FORMAT_GRAPH;
+		size = sizeof (guint32) * 5;
+	}
 
 	cur_block_maybe_expand (size);
 
 	cur_setnum (writer.cur_block, &writer.cur_pos, df);
+	if (g_id > 0) {
+		cur_setnum (writer.cur_block, &writer.cur_pos, g_id);
+	}
 	cur_setnum (writer.cur_block, &writer.cur_pos, s_id);
 	cur_setnum (writer.cur_block, &writer.cur_pos, p_id);
 	cur_setnum (writer.cur_block, &writer.cur_pos, o_id);
@@ -406,7 +436,8 @@ tracker_db_journal_append_delete_statement_id (gint s_id,
 }
 
 gboolean
-tracker_db_journal_append_insert_statement (gint         s_id,
+tracker_db_journal_append_insert_statement (gint         g_id,
+                                            gint         s_id,
                                             gint         p_id,
                                             const gchar *object)
 {
@@ -415,17 +446,26 @@ tracker_db_journal_append_insert_statement (gint         s_id,
 	gint size;
 
 	g_return_val_if_fail (writer.journal > 0, FALSE);
+	g_return_val_if_fail (g_id >= 0, FALSE);
 	g_return_val_if_fail (s_id > 0, FALSE);
 	g_return_val_if_fail (p_id > 0, FALSE);
 	g_return_val_if_fail (object != NULL, FALSE);
 
 	o_len = strlen (object);
-	df = 0x00;
-	size = (sizeof (guint32) * 3) + o_len + 1;
+	if (g_id == 0) {
+		df = 0x00;
+		size = (sizeof (guint32) * 3) + o_len + 1;
+	} else {
+		df = DATA_FORMAT_GRAPH;
+		size = (sizeof (guint32) * 4) + o_len + 1;
+	}
 
 	cur_block_maybe_expand (size);
 
 	cur_setnum (writer.cur_block, &writer.cur_pos, df);
+	if (g_id > 0) {
+		cur_setnum (writer.cur_block, &writer.cur_pos, g_id);
+	}
 	cur_setnum (writer.cur_block, &writer.cur_pos, s_id);
 	cur_setnum (writer.cur_block, &writer.cur_pos, p_id);
 	cur_setstr (writer.cur_block, &writer.cur_pos, object, o_len);
@@ -437,7 +477,8 @@ tracker_db_journal_append_insert_statement (gint         s_id,
 }
 
 gboolean
-tracker_db_journal_append_insert_statement_id (gint s_id,
+tracker_db_journal_append_insert_statement_id (gint g_id,
+                                               gint s_id,
                                                gint p_id,
                                                gint o_id)
 {
@@ -445,16 +486,25 @@ tracker_db_journal_append_insert_statement_id (gint s_id,
 	gint size;
 
 	g_return_val_if_fail (writer.journal > 0, FALSE);
+	g_return_val_if_fail (g_id >= 0, FALSE);
 	g_return_val_if_fail (s_id > 0, FALSE);
 	g_return_val_if_fail (p_id > 0, FALSE);
 	g_return_val_if_fail (o_id > 0, FALSE);
 
-	df = DATA_FORMAT_OBJECT_ID;
-	size = sizeof (guint32) * 4;
+	if (g_id == 0) {
+		df = DATA_FORMAT_OBJECT_ID;
+		size = sizeof (guint32) * 4;
+	} else {
+		df = DATA_FORMAT_OBJECT_ID | DATA_FORMAT_GRAPH;
+		size = sizeof (guint32) * 5;
+	}
 
 	cur_block_maybe_expand (size);
 
 	cur_setnum (writer.cur_block, &writer.cur_pos, df);
+	if (g_id > 0) {
+		cur_setnum (writer.cur_block, &writer.cur_pos, g_id);
+	}
 	cur_setnum (writer.cur_block, &writer.cur_pos, s_id);
 	cur_setnum (writer.cur_block, &writer.cur_pos, p_id);
 	cur_setnum (writer.cur_block, &writer.cur_pos, o_id);
@@ -510,7 +560,7 @@ tracker_db_journal_truncate (gsize new_size)
 }
 
 gboolean
-tracker_db_journal_commit_transaction (void)
+tracker_db_journal_commit_db_transaction (void)
 {
 	guint32 crc;
 	guint begin_pos;
@@ -659,6 +709,7 @@ tracker_db_journal_reader_shutdown (void)
 	reader.amount_of_triples = 0;
 	reader.type = TRACKER_DB_JOURNAL_START;
 	reader.uri = NULL;
+	reader.g_id = 0;
 	reader.s_id = 0;
 	reader.p_id = 0;
 	reader.o_id = 0;
@@ -739,8 +790,9 @@ tracker_db_journal_reader_next (GError **error)
 
 		/* Check the end is not before where we currently are */
 		if (reader.current >= reader.end) {
-			g_set_error (error, TRACKER_DB_JOURNAL_ERROR, 0, 
-			             "End of journal reached");
+			/* Return FALSE as there is no further entry but
+			 * do not set error as it's not an error case.
+			 */
 			return FALSE;
 		}
 
@@ -901,6 +953,23 @@ tracker_db_journal_reader_next (GError **error)
 				}
 			}
 
+			if (df & DATA_FORMAT_GRAPH) {
+				if (reader.end - reader.current < sizeof (guint32)) {
+					/* damaged journal entry */
+					g_set_error (error, TRACKER_DB_JOURNAL_ERROR, 0, 
+						     "Damaged journal entry, %d < sizeof(guint32)",
+						     (gint) (reader.end - reader.current));
+					return FALSE;
+				}
+
+				/* named graph */
+				reader.g_id = read_uint32 (reader.current);
+				reader.current += 4;
+			} else {
+				/* default graph */
+				reader.g_id = 0;
+			}
+
 			if (reader.end - reader.current < 2 * sizeof (guint32)) {
 				/* damaged journal entry */
 				g_set_error (error, TRACKER_DB_JOURNAL_ERROR, 0, 
@@ -984,7 +1053,8 @@ tracker_db_journal_reader_get_resource (gint         *id,
 }
 
 gboolean
-tracker_db_journal_reader_get_statement (gint         *s_id,
+tracker_db_journal_reader_get_statement (gint         *g_id,
+                                         gint         *s_id,
                                          gint         *p_id,
                                          const gchar **object)
 {
@@ -993,6 +1063,9 @@ tracker_db_journal_reader_get_statement (gint         *s_id,
 	                      reader.type == TRACKER_DB_JOURNAL_DELETE_STATEMENT,
 	                      FALSE);
 
+	if (g_id) {
+		*g_id = reader.g_id;
+	}
 	*s_id = reader.s_id;
 	*p_id = reader.p_id;
 	*object = reader.object;
@@ -1001,7 +1074,8 @@ tracker_db_journal_reader_get_statement (gint         *s_id,
 }
 
 gboolean
-tracker_db_journal_reader_get_statement_id (gint *s_id,
+tracker_db_journal_reader_get_statement_id (gint *g_id,
+                                            gint *s_id,
                                             gint *p_id,
                                             gint *o_id)
 {
@@ -1010,6 +1084,9 @@ tracker_db_journal_reader_get_statement_id (gint *s_id,
 	                      reader.type == TRACKER_DB_JOURNAL_DELETE_STATEMENT_ID,
 	                      FALSE);
 
+	if (g_id) {
+		*g_id = reader.g_id;
+	}
 	*s_id = reader.s_id;
 	*p_id = reader.p_id;
 	*o_id = reader.o_id;
