@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2009, Nokia (urho.konttori@nokia.com)
  *
@@ -18,8 +17,8 @@
  * Boston, MA  02110-1301, USA.
  */
 
-#ifndef __LIBTRACKERMINER_MANAGER_H__
-#define __LIBTRACKERMINER_MANAGER_H__
+#ifndef __LIBTRACKER_MINER_MANAGER_H__
+#define __LIBTRACKER_MINER_MANAGER_H__
 
 #include <glib-object.h>
 
@@ -53,17 +52,17 @@ struct TrackerMinerManagerClass {
 	GObjectClass parent_class;
 
 	void (* miner_progress) (TrackerMinerManager *manager,
-				 const gchar         *miner_name,
-				 const gchar         *status,
-				 gdouble              progress);
+	                         const gchar         *miner_name,
+	                         const gchar         *status,
+	                         gdouble              progress);
 	void (* miner_paused)   (TrackerMinerManager *manager,
-				 const gchar         *miner_name);
+	                         const gchar         *miner_name);
 	void (* miner_resumed)  (TrackerMinerManager *manager,
-				 const gchar         *miner_name);
+	                         const gchar         *miner_name);
 	void (* miner_activated)   (TrackerMinerManager *manager,
-				    const gchar         *miner_name);
+	                            const gchar         *miner_name);
 	void (* miner_deactivated) (TrackerMinerManager *manager,
-				    const gchar         *miner_name);
+	                            const gchar         *miner_name);
 };
 
 GType   tracker_miner_manager_get_type (void) G_GNUC_CONST;
@@ -74,35 +73,33 @@ GSList *tracker_miner_manager_get_running   (TrackerMinerManager *manager);
 GSList *tracker_miner_manager_get_available (TrackerMinerManager *manager);
 
 gboolean tracker_miner_manager_pause  (TrackerMinerManager *manager,
-				       const gchar         *miner,
-				       const gchar         *reason,
-				       guint32             *cookie);
+                                       const gchar         *miner,
+                                       const gchar         *reason,
+                                       guint32             *cookie);
 gboolean tracker_miner_manager_resume (TrackerMinerManager *manager,
-				       const gchar         *miner,
-				       guint32              cookie);
+                                       const gchar         *miner,
+                                       guint32              cookie);
 
 gboolean tracker_miner_manager_is_active (TrackerMinerManager *manager,
-					  const gchar         *miner);
+                                          const gchar         *miner);
 
 gboolean tracker_miner_manager_is_paused (TrackerMinerManager *manager,
-					  const gchar         *miner,
-					  GStrv               *applications,
-					  GStrv               *reasons);
+                                          const gchar         *miner,
+                                          GStrv               *applications,
+                                          GStrv               *reasons);
 
 gboolean tracker_miner_manager_get_status (TrackerMinerManager  *manager,
-					   const gchar          *miner,
-					   gchar               **status,
-					   gdouble              *progress);
-gboolean tracker_miner_manager_writeback  (TrackerMinerManager *manager,
-					   const gchar         *miner,
-					   const gchar        **subjects);
-
+                                           const gchar          *miner,
+                                           gchar               **status,
+                                           gdouble              *progress);
+gboolean tracker_miner_manager_ignore_next_update    (TrackerMinerManager *manager,
+                                                      const gchar         *miner,
+                                                      const gchar        **urls);
 const gchar * tracker_miner_manager_get_display_name (TrackerMinerManager *manager,
-						      const gchar         *miner);
+                                                      const gchar         *miner);
 const gchar * tracker_miner_manager_get_description  (TrackerMinerManager *manager,
-						      const gchar         *miner);
-
+                                                      const gchar         *miner);
 
 G_END_DECLS
 
-#endif /* __LIBTRACKERMINER_MANAGER_H__ */
+#endif /* __LIBTRACKER_MINER_MANAGER_H__ */
