@@ -34,20 +34,12 @@
 #include <glib/gstdio.h>
 
 #include <libtracker-common/tracker-file-utils.h>
-#include <libtracker-common/tracker-ontology.h>
 
-#include "tracker-main.h"
+#include <libtracker-extract/tracker-extract.h>
 
-
-#define NIE_PREFIX TRACKER_NIE_PREFIX
-#define NFO_PREFIX TRACKER_NFO_PREFIX
-#define NCO_PREFIX TRACKER_NCO_PREFIX
-
-#define RDF_PREFIX TRACKER_RDF_PREFIX
-#define RDF_TYPE RDF_PREFIX "type"
-
-static void extract_abw (const gchar *uri,
-                         TrackerSparqlBuilder   *metadata);
+static void extract_abw (const gchar          *uri,
+                         TrackerSparqlBuilder *preupdate,
+                         TrackerSparqlBuilder *metadata);
 
 static TrackerExtractData data[] = {
 	{ "application/x-abiword", extract_abw },
@@ -55,8 +47,9 @@ static TrackerExtractData data[] = {
 };
 
 static void
-extract_abw (const gchar *uri,
-             TrackerSparqlBuilder   *metadata)
+extract_abw (const gchar          *uri,
+             TrackerSparqlBuilder *preupdate,
+             TrackerSparqlBuilder *metadata)
 {
 	FILE *f;
 	gchar *filename;
@@ -73,7 +66,6 @@ extract_abw (const gchar *uri,
 		line = NULL;
 		length = 0;
 
-		tracker_sparql_builder_subject_iri (metadata, uri);
 		tracker_sparql_builder_predicate (metadata, "a");
 		tracker_sparql_builder_object (metadata, "nfo:Document");
 
@@ -121,7 +113,7 @@ extract_abw (const gchar *uri,
 }
 
 TrackerExtractData *
-tracker_get_extract_data (void)
+tracker_extract_get_data (void)
 {
 	return data;
 }
