@@ -26,12 +26,7 @@
 
 #include <libxml/HTMLparser.h>
 
-#include <libtracker-common/tracker-ontologies.h>
-
 #include <libtracker-extract/tracker-extract.h>
-#include <libtracker-extract/tracker-utils.h>
-
-#define RDF_TYPE TRACKER_RDF_PREFIX "type"
 
 typedef enum {
 	READ_TITLE,
