@@ -27,9 +27,9 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
-#include <panel-applet-gconf.h>
+#include <panel-applet.h>
 
-#include <libtracker-client/tracker.h>
+#include <libtracker-client/tracker-client.h>
 
 #include "tracker-results-window.h"
 #include "tracker-aligned-window.h"
