@@ -54,8 +54,14 @@ const TestInfo tests[] = {
 	{ "algebra/filter-nested-2", "algebra/data-1", FALSE },
 	{ "algebra/filter-scope-1", "algebra/data-2", FALSE },
 	{ "algebra/var-scope-join-1", "algebra/var-scope-join-1", FALSE },
+	{ "anon/query", "anon/data", FALSE },
+	{ "basic/base-prefix-3", "basic/data-1", FALSE },
+	{ "basic/predicate-variable", "basic/data-1", FALSE },
 	{ "bnode-coreference/query", "bnode-coreference/data", FALSE },
 	{ "bound/bound1", "bound/data", FALSE },
+	{ "datetime/delete-1", "datetime/data-3", FALSE },
+	{ "datetime/functions-localtime-1", "datetime/data-1", FALSE },
+	{ "datetime/functions-timezone-1", "datetime/data-2", FALSE },
 	{ "expr-ops/query-ge-1", "expr-ops/data", FALSE },
 	{ "expr-ops/query-le-1", "expr-ops/data", FALSE },
 	{ "expr-ops/query-minus-1", "expr-ops/data", FALSE },
@@ -65,8 +71,15 @@ const TestInfo tests[] = {
 	{ "expr-ops/query-unplus-1", "expr-ops/data", FALSE },
 	{ "functions/functions-property-1", "functions/data-1", FALSE },
 	{ "functions/functions-tracker-1", "functions/data-1", FALSE },
+	{ "functions/functions-tracker-2", "functions/data-2", FALSE },
+	{ "functions/functions-tracker-loc-1", "functions/data-3", FALSE },
 	{ "functions/functions-xpath-1", "functions/data-1", FALSE },
 	{ "functions/functions-xpath-2", "functions/data-1", FALSE },
+	{ "functions/functions-xpath-3", "functions/data-1", FALSE },
+	{ "functions/functions-xpath-4", "functions/data-1", FALSE },
+	{ "graph/graph-1", "graph/data-1", FALSE },
+	{ "graph/graph-2", "graph/data-2", FALSE },
+	{ "graph/graph-3", "graph/data-3", FALSE },
 	{ "optional/q-opt-complex-1", "optional/complex-data-1", FALSE },
 	{ "regex/regex-query-001", "regex/regex-data-01", FALSE },
 	{ "regex/regex-query-002", "regex/regex-data-01", FALSE },
@@ -74,7 +87,7 @@ const TestInfo tests[] = {
 	{ "sort/query-sort-2", "sort/data-sort-1", FALSE },
 	{ "sort/query-sort-3", "sort/data-sort-3", FALSE },
 	{ "sort/query-sort-4", "sort/data-sort-4", FALSE },
-	{ "sort/query-sort-5", "sort/data-sort-4", FALSE }, 
+	{ "sort/query-sort-5", "sort/data-sort-4", FALSE },
 	{ "subqueries/subqueries-1", "subqueries/data-1", FALSE },
 	/* Bracket error after WHERE */
 	{ "error/query-error-1", "error/query-error-1", TRUE },
@@ -116,16 +129,34 @@ test_sparql_query (gconstpointer test_data)
 
 	tracker_data_manager_init (TRACKER_DB_MANAGER_FORCE_REINDEX,
 	                           data_prefix,
-	                           NULL, NULL);
+	                           NULL, FALSE);
 
 	/* data_path = g_build_path (G_DIR_SEPARATOR_S, TOP_SRCDIR, "tests", "libtracker-data", NULL); */
 
 	/* load data set */
 	data_filename = g_strconcat (data_prefix, ".ttl", NULL);
-	tracker_data_begin_transaction ();
-	tracker_turtle_reader_load (data_filename, &error);
-	tracker_data_commit_transaction ();
-	g_assert_no_error (error);
+	if (g_file_test (data_filename, G_FILE_TEST_IS_REGULAR)) {
+		tracker_data_begin_db_transaction ();
+		tracker_turtle_reader_load (data_filename, &error);
+		tracker_data_commit_db_transaction ();
+		g_assert_no_error (error);
+	} else {
+		/* no .ttl available, assume .rq with SPARQL Update */
+		gchar *data;
+
+		g_free (data_filename);
+
+		data_filename = g_strconcat (data_prefix, ".rq", NULL);
+		g_file_get_contents (data_filename, &data, NULL, &error);
+		g_assert_no_error (error);
+
+		tracker_data_begin_db_transaction ();
+		tracker_data_update_sparql (data, &error);
+		tracker_data_commit_db_transaction ();
+		g_assert_no_error (error);
+
+		g_free (data);
+	}
 
 	query_filename = g_strconcat (test_prefix, ".rq", NULL);
 	g_file_get_contents (query_filename, &query, NULL, &error);
